###################################################################
#								  #
#		    SSSS       M       M			  #
#		   S    S      MM     MM			  #
#		   S	       M M   M M			  #
#		    SSSS       M  M M  M			  #
#		        S      M   M   M			  #
#		   S    S      M       M			  #
#		    SSSS       M       M			  #
#								  #
#								  #
###################################################################
#
# This makefile makes various standalone programmes, and then
# SM itself, in subdirectory src. See the makefile in src
# and (especially) the file options.h to modify the ways that SM is compiled.
# You may well be able to set these with the set_opts script in this directory
#
# If you change (e.g.) the value of CC in this Makefile, you must 
# always run always make from this directory or from ./src
#
.c.o :
	$(CC) -c $(CFLAGS) $*.c
#
# Which compiler to use.
#
#  If you are using gcc -Wall the compilation of bison will produce many
# warnings. You may ignore these, or move the -Wall flag into CFLAGS.
#  When compiling control.y you will get two from bison.simple.
#  Compiling SM itself should produce a single complaint from each main
# programme, as gcc doesn't understand exit().
#
CC = gcc -Wall -Wpointer-arith -funsigned-char
#
# cc flags
#
CFLAGS = -g -Isrc
#
# Installation directories:
#	DESTDIR : binaries (sm and rasterise)
#	DESTLIB: libraries (libdevices.a, libplotsub.a, and libutils.a)
#	DESTETC: (miscellaneous files, and macro and help directories)
#	DESTMAN: manual page(s)
#
DEST = /usr/local
DESTDIR = $(DEST)/bin
DESTLIB = $(DEST)/lib
DESTETC = $(DESTLIB)/sm
DESTINFO = $(DEST)/info
DESTMAN = $(DEST)/man/man1
#
# Path to search for .sm files
#
DEFAULT_PATH =". ~ $(DESTETC)/"
#
# linker flags
#
LDFLAGS = -lm
#
# This seems to be needed by a convex
#
.IF $(MAKECMD) != dmake
MAKE = make
.END
#
# If you don't need ranlib comment out the definition
#
RANLIB = /usr/bin/ranlib
#
# Don't change this, please
#
SHELL = /bin/sh
#
# Make all targets (should be first in Makefile)
#
all : notice sm fonts misc
notice :
	@- echo ""
	@- echo SM is copyright 1987, 1989, 1990, 1992, 1993 Robert Lupton and Patricia Monger.
	@- echo You may not distribute either source or executables without
	@- echo specific permission from the authors. See the file copyright.h
	@- echo for more details.
	@- echo ""
#
# Preserve these files
#
.PRECIOUS : fonts.bin
#
# Make SM
#
sm :
	( cd src ; \
	  $(MAKE) $(MFLAGS) "CC=$(CC)" "CFLAGS=$(CFLAGS)" "RANLIB=$(RANLIB)" \
			DEFAULT_PATH=$(DEFAULT_PATH) all_src )
#
# copy executable to current release version.
#
install :
	@- echo Installing SM
	@ for f in $(DESTDIR) $(DESTLIB) $(DESTETC) $(DESTMAN); do \
		fd=`echo $$f | sed -e 's#/[^/]*$$##'`; \
		if test ! -d $$fd ; then \
			echo creating $$fd; mkdir $$fd; \
		fi ; \
		if test ! -d $$f ; then \
			echo creating $$f; mkdir $$f; \
		fi \
	done
	cp src/sm rasterise $(DESTDIR)
	cp src/libdevices.a src/libplotsub.a src/libutils.a $(DESTLIB)
	if test -f src/libparser.a; then cp src/libparser.a $(DESTLIB); fi
	if test "$(RANLIB)" != "" -a -f "$(RANLIB)"; then \
		(	cd $(DESTLIB); \
			$(RANLIB) libdevices.a libplotsub.a libutils.a; \
			if test -f libparser.a; then $(RANLIB) libparser.a; fi\
		); fi
	cp filecap graphcap fonts.bin $(DESTETC)
	if test ! -f /etc/termcap; then cp termcap $(DESTETC); fi
	for f in macro help; do \
		if test ! -d $(DESTETC)/$$f ; then \
			mkdir $(DESTETC)/$$f ; \
		fi ; \
		cp `find $$f ! -type d -print` $(DESTETC)/$$f ; \
	done
	cp sm.man $(DESTMAN)/sm.1
	if test ! -d $(DESTETC) ; then mkdir $(DESTETC); fi
	@- if test -f $(DESTETC)/.sm; then \
		echo File $(DESTETC)/.sm exists\; I am not modifying it; \
	   else \
		( \
		echo "fonts		$(DESTETC)/fonts.bin" ;\
		echo "history_file	.smhist" ;\
		echo "history		80" ;\
		echo "temp_dir	/tmp/" ;\
		for f in filecap graphcap ; do \
			echo "$$f		$(DESTETC)/$$f" ;\
		done;\
		if test ! -f /etc/termcap; then \
			echo "termcap		$(DESTETC)/termcap" ;\
		fi; \
		for f in help macro ; do \
			echo "$$f		$(DESTETC)/$$f/" ;\
		done;\
		) > $(DESTETC)/.sm; \
	   fi
installinfo :
	( cd info ; $(MAKE) $(MFLAGS) DESTINFO=$(DESTINFO) installinfo )
#
# Now various stand-alone programmes:
#
# Convert the fonts to binary format
#
read_fonts : read_fonts.o
	$(CC) -o read_fonts read_fonts.o -lm
#
fonts : fonts.bin
fonts.bin : read_fonts hershey_oc.dat font_index tex.def
	 ./read_fonts hershey_oc.dat font_index tex.def fonts.bin
#
# Various things need compiling:
#
misc : rasterise
	@- sh config $(CC) $(CFLAGS)
#
# Bootstrap compile_g without a valid cacheg.dat file
# (not made by default)
#
boot_g : compile_g.o src/tty.h
	$(CC) -DBOOTSTRAP -o boot_g -Isrc compile_g.c
#
# The programme to compile graphcap entries
#
compile_g.o : src/dummies.h src/tty.h
compile_g : compile_g.o src/libdevices.a src/libutils.a 
	$(CC) -o compile_g compile_g.o -Lsrc -ldevices -lutils $(LDFLAGS)
#
# Extract the grammar from a yacc/bison file
#
get_grammar : get_grammar.o
	$(CC) -o get_grammar get_grammar.o
#
# Rasterise vector files produced by the "raster" device
#
rasterise.o : src/dummies.h
rasterise : rasterise.o src/libdevices.a src/libutils.a
	$(CC) -o rasterise rasterise.o -Lsrc -ldevices -lutils $(LDFLAGS)
#
# Here is the library to allow a programme to call the SM parser. It
# isn't built by default
#
Parser : 
	( cd src ; \
	  $(MAKE) $(MFLAGS) "CC=$(CC)" "CFLAGS=$(CFLAGS)" "RANLIB=$(RANLIB)" \
			DEFAULT_PATH=$(DEFAULT_PATH) libparser.a )
#
# prepare a distribution of SM
#
distrib :
	@ echo -n "Converting makefiles:"
	@ for f in . src src/bison src/devices src/plotsub src/utils; do \
		echo -n " $$f"; \
		if [ -f $$f/makefile.mk ]; then \
			cp $$f/makefile.mk $$f/Makefile ;\
		fi ;\
		if [ \! -f $$f/makefile.mk ]; then \
			cp $$f/Makefile $$f/makefile.mk ;\
		fi ;\
		./dmake_to_make < $$f/Makefile > $$f/Makefile.new ;\
		mv $$f/Makefile $$f/Makefile.orig ;\
		mv $$f/Makefile.new $$f/Makefile ;\
	done
	@ echo ""
	@ echo Removing CC from Makefile
	@ sed -e 's/^CC *=.*/CC = cc/' < Makefile > Makefile~
	@ mv Makefile~ Makefile
	@ echo Adding copyright notice to and removing devices from options.h
	@ ./fix_options < src/options.h > src/options.hh~
	@ mv src/options.hh~ src/options.h
	@ echo Removing devices from src/Makefile
	@ sed -e 's/^LDFLAGS.*/LDFLAGS = -lm/' < src/Makefile > src/Makefile~
	@ mv src/Makefile~ src/Makefile
	@ echo "Making sm.install from info/install.txi"
	@ ( cd info ; make ../sm.install ) > /dev/null
	@ find . \( -name CVS -prune -o -name .cvsignore \) -exec rm -rf {} \;
	@ rm -f dmake_to_make
	@ /bin/cp /dev/null .config
	@ echo You may now want to \"make clean\"
#
# clean out all object code and junk files
#
empty : clean
	- rm -f fonts.bin read_fonts rasterise compile_g boot_g
	- (cd src ; $(MAKE) empty )
	- (cd info ; $(MAKE) empty )
clean : tidy
	- rm -f *.o patch/*.o get_grammar
	- (cd src ; $(MAKE) clean )
	- (cd callable ; $(MAKE) clean )
	- (cd info ; $(MAKE) clean )
#
tidy :
	- rm -f *.ckp core *~ .*~ *.orig *.rej
	- (cd src ; $(MAKE) tidy )
	- rm -f doc/*.log doc/*.dvi doc/*~ doc/*.orig doc/*.rej \
		doc/index1.tex doc/contents1.tex \
		help/*~ help/*.orig help/*.rej \
		macro/*~ macro/*.orig macro/*.rej \
		patch/*~ patch/*.orig patch/*.rej
	- (cd info ; $(MAKE) tidy )
#
# Here are all the dependencies:
#
#START_DEPEND
read_fonts.o :	src/fonts.h
read_fonts.o :	src/options.h
