/*
 * X11 device driver for SM.
 *
 * If you define DISABLE_BACKING to cc the code will not use
 * a backing store, even if it is supported by the server
 */
#include "copyright.h"
#include "options.h"
#if defined(X11)
#ifdef SYS_V
#  define SYSV				/* needed by X */
#endif
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/cursorfont.h>
#ifdef hpux
#define _STRING_INCLUDED
#endif
#include <X11/Xos.h>
#if defined(NODEVICE)			/* X on an IBM/6000 defines it */
#  undef NODEVICE
#endif
#include "sm.h"
#include "devices.h"
#include "sm_declare.h"

#if !defined(FORCE_BACKING_STORE) && !defined(NO_SELECT)
#  define DISABLE_BACKING
#endif

#define NARG 20
#define NVEC 200                /* size of xvec */
#if 0
#  define GetForeground(GC) ((GC)->values.foreground)  /* should be provided!*/
#  define XresourceManagerString(D) (D->xdefaults) /* doesn't this exist? */
#endif

static char *get_resource();
static Colormap cmap;
static Cursor graphics_cursor;		/* cursor for cursor */
static Display *display = NULL;
extern float xscreen_to_pix,yscreen_to_pix;
static Font font = (Font)(-1);
static GC erase_gc,			/* Graphics Context for erasing */
          gc,                           /* GC for drawing */
          redraw_gc;			/* GC for redrawing */
static int backpix,                     /* pixel value for background */
	   get_foreground(),		/* get the current foreground */
  	   have_backing,		/* server supports backing store */
           h_font,w_font,               /* height and (mean) width of chars */
           nvec,                        /* number of vectors drawn */
	   X_get_opts();
static unsigned int depth,		/* depth of display */
	            height = 0,		/* height of window */
		    width;		/* width of window */
static long event_mask;			/* which events do I want to see? */
static Pixmap backing;			/* backing for redrawing window */
static void size_x11(),
	    usage(),
	    x_clear_pixmap();
static Window root,			/* root window */
	      wind = 0;			/* graphics window */
static XEvent *_x11_redraw();
static int hand_XError(),
	   hand_XIOError();
static XrmOptionDescRec options[] = {
   {"#",		".icongeometry", XrmoptionStickyArg,	NULL},
   {"=",		".geometry",	XrmoptionStickyArg,	NULL},
   {"-borderwidth",	".borderwidth",	XrmoptionSepArg,	NULL},
   {"-bd",		".borderwidth",	XrmoptionSepArg,	NULL},
   {"-background",	".background",	XrmoptionSepArg,	NULL},
   {"-bg",		".background",	XrmoptionSepArg,	NULL},
   {"-display",		".display",	XrmoptionSepArg,	NULL},
   {"-foreground",	".foreground",	XrmoptionSepArg,	NULL},
   {"-fg",		".foreground",	XrmoptionSepArg,	NULL},
   {"-fn",		".font",	XrmoptionSepArg,	NULL},
   {"-font",		".font",	XrmoptionSepArg,	NULL},
   {"-geometry",	".geometry",	XrmoptionSepArg,	NULL},
   {"-iconic",		".iconic",	XrmoptionNoArg,		"1"},
   {"-name",		".name",	XrmoptionSepArg,	NULL},
   {"-preopened",	".preopened",	XrmoptionSepArg,	NULL},
   {"-synchronise",	".synchronise",	XrmoptionNoArg,		"1"},
   {"-title",		".title",	XrmoptionSepArg,	NULL},
};
#define NOPT (sizeof(options)/sizeof(options[0]))
static XSegment *xvec;

int
x11_setup(s)
char *s;
{
   char *backcolor;			/* name of background colour */
   char *fontname;			/* name of font */
   char *forecolor;			/* name of foreground colour */
   char *name;				/* name of application */
   char *ptr;
   char *title;				/* title of application */
   int border_width,			/* as it says */
       iconic = 0,			/* start iconised? */
       icon_x,icon_y,			/* offset of icon */
       icon_position = 0,		/* is icon position specified? */
       root_height,			/* height of display */
       root_width,			/* width of display */
       screen,				/* screen in use */
       sync = 0,			/* Synchronise Errors */
       xoff,yoff,			/* position of window */
       x_position = 0;			/* is window position specified? */
   unsigned int xs = 512,ys = 512;	/* size of window */
   unsigned long mask;                  /* mask for attributes */
   Pixmap icon_pixmap;
   static unsigned char gray_bits[32] = {
                                  /* A 16*16 array of bits, in the pattern: */
      0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55,   /* 1010101010101010 */
      0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55,   /* 0101010101010101 */
      0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55,   /* 1010101010101010 */
      0xaa, 0xaa, 0x55, 0x55, 0xaa, 0xaa, 0x55, 0x55};  /* 0101010101010101 */
   static char unsigned icon_bitmap[] = {
#     define X11_ICON
#     include "SM_icon.h"
   };
   XColor cdef;
   XEvent event;
   XFontStruct *font_info;
   XGCValues gc_values;
   static XrmDatabase rdb = NULL;	/* resources from args and xrdb */
   XSetWindowAttributes win_atts;       /* struct to g/set window attributes */
   XSizeHints hints;                    /* hints for window manager */
   XWMHints wmhints;			/* More hints for WM */
   
   if(*s != '\0' && match_pattern(s,"-h.*",(char **)NULL) != NULL) {
      usage();
      return(-1);
   }
   
   if(wind == 0) {
      XSetErrorHandler(hand_XError);
      XSetIOErrorHandler(hand_XIOError);

      if(X_get_opts(s,&rdb) < 0) {
	 return(-1);
      }

      border_width = atoi(get_resource(rdb,"borderwidth","4"));
      backcolor = get_resource(rdb,"background","");
      forecolor = get_resource(rdb,"foreground","white");
      iconic = atoi(get_resource(rdb,"iconic","0"));
      name = get_resource(rdb,"name",PROGNAME);
      sync = atoi(get_resource(rdb,"synchronise","0"));
      title = get_resource(rdb,"title","SM");
/*
 * Now window and icon geometry
 */
      mask = XParseGeometry(get_resource(rdb,"geometry",""),
			    &xoff,&yoff,&xs,&ys);
      if((mask & XValue) && (mask & YValue)) x_position = 1;
      if(xs == 0) xs = 1;
      if(ys == 0) ys = 1;

      if(*(ptr = get_resource(rdb,"icongeometry","")) != '\0') {
	 unsigned int h,w;	/* not used */

	 mask = XParseGeometry(ptr,&icon_x,&icon_y,&w,&h);
	 if((mask & XValue) && (mask & YValue)) {
	    icon_position = 1;
	 }
      }
/*
 * Finished dealing with user options and argument string
 */
      screen = DefaultScreen(display);
      if(sync) {
	 XSynchronize(display,1);
      }
      cmap = DefaultColormap(display,screen);
      depth = DisplayPlanes(display,screen);
      root = RootWindow(display,screen);
      root_height = DisplayHeight(display,screen);
      root_width = DisplayWidth(display,screen);
/*
 * try to get the background colour, but don't allocate a r/w cell for it
 */
      backpix = BlackPixel(display,screen);
      if(*backcolor != '\0' || (backcolor = get_val("background")) != NULL) {
         int i;

         if((i = XParseColor(display,cmap,backcolor,&cdef)) == 0) {
            int r,g,b;                  /* failed - do it ourselves */
            if(parse_color(backcolor,&r,&g,&b) >= 0) {
               cdef.red = (r << 8);
               cdef.green = (g << 8);
               cdef.blue = (b << 8);
               i = 1;
            } else {
	       msg_1s("I don't understand the colour \"%s\" -- using black\n",
		      backcolor);
	    }
         }
         if(i != 0 && XAllocColor(display,cmap,&cdef)) {
	    backpix = cdef.pixel;
	 }
      }
#ifdef DISABLE_BACKING
      have_backing = 0;
#else
      have_backing = (DoesBackingStore(display->screens) == Always);
#endif

      graphics_cursor = XCreateFontCursor(display,XC_crosshair);
      icon_pixmap = XCreateBitmapFromData(display,root,(char *)icon_bitmap,
					  SM_ICON_XSIZE,SM_ICON_YSIZE);

      mask = 0;
      win_atts.background_pixel = backpix;
      mask |= CWBackPixel;
      win_atts.border_pixmap =
	XCreatePixmapFromBitmapData(display,root,(char *)gray_bits,16,16,
		  BlackPixel(display,screen),WhitePixel(display,screen),depth);
      mask |= CWBorderPixmap;
      if(have_backing) {
	 win_atts.backing_store = Always;
	 mask |= CWBackingStore;
      }
/*
 * Finally create the window!
 */
      hints.flags = 0;
      hints.width = xs;
      hints.height = ys;
      hints.flags |= PSize;
      if(x_position) {
	 if(xoff < 0) {
	    xoff += root_width - 1 - hints.width - 2*border_width;
	 }
	 if(yoff < 0) {
	    yoff += root_height - 1 - hints.height - 2*border_width;
	 }
	 hints.x = xoff;
	 hints.y = yoff;
	 hints.flags |= USPosition;
      } else {
	 hints.x = hints.y = 100;
      }
      hints.min_width = hints.min_height = 100;
      hints.flags |= PMinSize;

      if(wind == 0 &&
          (wind = XCreateWindow(display,root,hints.x,hints.y,
		     hints.width,hints.height,border_width,depth,
                        InputOutput,CopyFromParent,mask,&win_atts)) == 0) {
         msg_1s("Can't open display %s\n",XDisplayName(s));
         return(-1);
      }

      XSetStandardProperties(display,wind,name,name,None,
			                             (char **)NULL,0,&hints);

      wmhints.flags = 0;
      wmhints.initial_state = iconic ? IconicState : NormalState;
      wmhints.flags |= StateHint;
      wmhints.icon_pixmap = icon_pixmap;
      wmhints.flags |= IconPixmapHint;
      wmhints.input = True;
      wmhints.flags |= InputHint;
      if(icon_position) {
	 if(icon_x < 0) {
	    icon_x += root_width - 1 - SM_ICON_XSIZE - 2*2;
	    				/* guess that icon border width is 2 */
	 }
	 if(icon_y < 0) {
	    icon_y += root_height - 1 - SM_ICON_YSIZE - 2*2;
	 }
	 wmhints.icon_x = icon_x;
	 wmhints.icon_y = icon_y;
	 wmhints.flags |= IconPositionHint;
      }
      XSetWMHints(display,wind,&wmhints);

      XStoreName(display,wind,title);
      event_mask = ExposureMask | StructureNotifyMask;
      XSelectInput(display,wind,event_mask);
/*
 * We can now finish attaching attributes to the window
 * (some we needed at its creation)
 */
#if defined(VMS)
      fontname = "fixed";
#else
      fontname = "8x13";
#endif
	
      fontname = get_resource(rdb,"font",fontname);
      if((font_info = XLoadQueryFont(display, fontname)) == NULL) {
	 msg_1s("Can't find font %s\n",fontname);
      } else {
         h_font = font_info->max_bounds.ascent;
         w_font = font_info->max_bounds.width;
         font = font_info->fid;
      }

      mask = 0;
      gc_values.fill_style = FillSolid;
      mask |= GCFillStyle;
      gc_values.foreground = WhitePixel(display,screen);
      default_ctype(forecolor);           /* set_dev()'ll look in .sm */
      mask |= GCForeground;
      gc_values.background = backpix;
      mask |= GCBackground;
      gc_values.font = font;
      mask |= GCFont;

      gc = XCreateGC(display,root,mask,&gc_values);
      redraw_gc = XCreateGC(display,root,mask,&gc_values);
      gc_values.foreground = backpix;
      erase_gc = XCreateGC(display,root,mask,&gc_values);
/*
 * create display list for vectors
 */
      if((xvec = (XSegment *)malloc(NVEC*sizeof(XSegment))) == NULL) {
         msg("Can't allocate vectors in x11_setup\n");
         return(-1);
      }
      nvec = 0;
/*
 * Put up window for the first time
 */
      XMapWindow(display,wind);
      if(!iconic) {
	 XMaskEvent(display,StructureNotifyMask,&event); /* watch creation */
	 XPutBackEvent(display,&event);			/* and reschedule it */
      }
   } else {
      forecolor = get_resource(rdb,"foreground","white");
      default_ctype(forecolor);
   }
   
   XRaiseWindow(display,wind);
   size_x11();
   _x11_redraw();

   return(0);
}

/****************************************************************/

int
x11_char(s,x,y)
char *s;
int x,y;
{
   int len;
   
   if(s == NULL) {
      if(aangle == 0.0 && eexpand == 1.0) {
	 cheight = h_font/yscreen_to_pix;
	 cwidth = w_font/xscreen_to_pix;
      }
      return(0);
   }
   x = x*xscreen_to_pix;
   y = height - (int)(y*yscreen_to_pix) - 1;
   len = strlen(s);

   XDrawString(display,wind,gc,x,y,s,len);
   if(!have_backing) {
      XDrawString(display,backing,gc,x,y,s,len);
   }
   return(0);
}
 
/****************************************************************/

void
x11_close()
{
   ;
}
 
/****************************************************************/

int
x11_cursor(x,y)
int *x,*y;
{
   char buff[5];
   int kx,ky,				/* cursor position in keyboard wind */
       rx,ry;				/* cursor position in root wind */
   static int cx = -1,cy;		/* last position of cursor in wind */
   unsigned int button_status;		/* not used */
   Window child_wind,			/* the values of these Windows */
          root_wind;			/* 	          are not used */
   XEvent event;

   XQueryPointer(display,root,&root_wind,&child_wind,&rx,&ry,
						     &kx,&ky,&button_status);
   XDefineCursor(display,wind,graphics_cursor);

   if(cx < 0) {
      cx = width/2;
      cy = height/2;
   }
   XWarpPointer(display,None,wind,0,0,0,0,cx,cy);

   while(XCheckTypedEvent(display,ButtonPress|KeyPress,&event)) {
      continue;				/* throw away old key/button events */
   }
   event_mask |= ButtonPressMask|KeyPressMask;
   XSelectInput(display,wind,event_mask);
   for(;;) {
      XNextEvent(display,&event);
      XPutBackEvent(display,&event);	/* we are sure that there's something*/
      event = *_x11_redraw();		/* wait for desired events */
      if(event.type == ButtonPress || event.type == KeyPress) {
	 break;
      }
   }
   event_mask &= ~(ButtonPressMask|KeyPressMask);
   XSelectInput(display,wind,event_mask);

   XUndefineCursor(display,wind);
   XQueryPointer(display,wind,&root_wind,&child_wind,&rx,&ry,
		 				     &cx,&cy,&button_status);
   XWarpPointer(display,None,root,0,0,0,0,kx,ky);
   XFlush(display);			/* make the pointer move NOW */

   *x = cx/xscreen_to_pix + 0.5;
   *y = (height - cy - 1)/yscreen_to_pix + 0.5;

   if(event.type == ButtonPress) {
      switch (event.xbutton.button) {
       case Button1:
	 return('e');
       case Button2:
	 return('p');
       case Button3:
	 return('q');
       default:
	 return('p');
      }
   }

   XLookupString((XKeyEvent *)&event,buff,sizeof(buff),(KeySym *)NULL,
		 				(XComposeStatus *)NULL);
   return(buff[0]);
}

/****************************************************************/

void
x11_draw(x,y)
int x,y;
{
   x11_line(xp,yp,x,y);
}

/****************************************************************/

void
x11_erase()
{
   nvec = 0;
   XClearWindow(display,wind);
   if(!have_backing) {
      x_clear_pixmap(display,backing);
   }
   XRaiseWindow(display,wind);
}

/****************************************************************/

int
x11_fill_pt(n)
int n;                          /* number of sides */
{
   float dtheta, theta;
   static float old_ang;	/* old values of angle */
   int i,
       xpsize,			/* scale for points */
       ypsize;
   static int num = -1,         /* number of vertices used last time */
	      old_xp,old_yp;	/* old values of xpsize, ypsize */
   XPoint vlist[MAX_POLYGON + 1];	/* corners of point */
   static XPoint vlist0[MAX_POLYGON + 1]; /* corners of point at (0,0) */

   if(n < 2) {
      x11_line(xp,yp,xp,yp);
      return(0);
   }

   dtheta = 2*PI/n;
   xpsize = ypsize = PDEF*eexpand;
   if(aspect > 1.0) {                   /* allow for aspect ratio of screen */
      ypsize /= aspect;
   } else if(aspect < 1.0) {
      xpsize *= aspect;
   }

   if(n != num || aangle != old_ang || xpsize != old_xp || ypsize != old_yp) {
      if(n > MAX_POLYGON) num = MAX_POLYGON;
      else num = n;

      theta = 3*PI/2 + dtheta/2 + aangle*PI/180;

      old_ang = aangle;
      old_xp = xpsize;
      old_yp = ypsize;

      for(i = 0;i < num;i++) {
	 vlist0[i].x = xscreen_to_pix*xpsize*cos(theta) + 0.5;
	 vlist0[i].y = yscreen_to_pix*(SCREEN_SIZE - ypsize*sin(theta)) + 0.5;
	 vlist[i].x = vlist0[i].x + xscreen_to_pix*xp;
	 vlist[i].y = vlist0[i].y - yscreen_to_pix*yp;
	 theta += dtheta;
      }
   } else {				/* no need for more trig. */
      for(i = 0;i < num;i++) {
	 vlist[i].x = vlist0[i].x + xscreen_to_pix*xp;
	 vlist[i].y = vlist0[i].y - yscreen_to_pix*yp;
      }
   }
   vlist[i].x = vlist[0].x;
   vlist[i].y = vlist[0].y;
/*
 * We need the bounding polygon as well as fill to get all of the edges
 */
   XFillPolygon(display,wind,gc,vlist,num,Convex,CoordModeOrigin);
   XDrawLines(display,wind,gc,vlist,num + 1,CoordModeOrigin);
   if(!have_backing) {
      XFillPolygon(display,backing,gc,vlist,num,Convex,CoordModeOrigin);
      XDrawLines(display,backing,gc,vlist,num + 1,CoordModeOrigin);
   }
   return(0);
}

/****************************************************************/

int
x11_dot(x,y)
int x,y;
{
   x11_line(x - (int)(llweight/(2*xscreen_to_pix)),y,
	    x + (int)(llweight/(2*xscreen_to_pix)),y);
   return(0);
}

/****************************************************************/

void
x11_gflush()
{
   XDrawSegments(display,wind,gc,xvec,nvec);
   if(!have_backing) {
      XDrawSegments(display,backing,gc,xvec,nvec);
   }
   XFlush(display);
   nvec = 0;
}

/****************************************************************/

void
x11_idle()
{
   ;
}

/****************************************************************/

void
x11_line(x1,y1,x2,y2)
int x1,y1,x2,y2;
{
   x1 = x1*xscreen_to_pix;
   x2 = x2*xscreen_to_pix;
   y1 = height - (int)(y1*yscreen_to_pix) - 1;
   y2 = height - (int)(y2*yscreen_to_pix) - 1;
   xvec[nvec].x1 = x1;
   xvec[nvec].y1 = y1;
   xvec[nvec].x2 = x2;
   xvec[nvec].y2 = y2;
   nvec++;
   if(nvec > NVEC - 1) {
      XDrawSegments(display,wind,gc,xvec,nvec);
      if(!have_backing) {
	 XDrawSegments(display,backing,gc,xvec,nvec);
      }
      nvec = 0;
   }
}

/****************************************************************/
/*
 * Only support erasing of lines (lt 10 and 11)
 */

int
x11_ltype(lt)
int lt;
{
   static int old_forepix = -1;

   if(lt == 10) {
      if(get_foreground(gc) != backpix) {
	 old_forepix = get_foreground(gc);
      }
      XSetForeground(display,gc,backpix);
      return(0);
   } else if(lt == 11) {
      XSetForeground(display,gc,old_forepix);
      return(0);
   }
   return(-1);
}

/****************************************************************/

int
x11_lweight(lw)
double lw;
{
   XGCValues gc_values;
   
   if(lw > 1.0) {
      lw--;				/* lw == 0 is special */
   }

   gc_values.line_width = lw + 0.5;
   XChangeGC(display,gc,GCLineWidth,&gc_values);
   return(0);
}

/****************************************************************/

void
x11_page()
{
   XRaiseWindow(display,wind);
   size_x11();
   _x11_redraw();
}

/****************************************************************/

void
x11_redraw(fd)
int fd;					/* f.d. that get1char is waiting on */
{
#if defined(NO_SELECT)
   _x11_redraw();
#else
   int mask;				/* mask of descriptors to check */
   int ncheck;				/* number of descriptors to check */
   int readmask;			/* the desired mask */
   int xfd;				/* x11's file descriptor */

   if(fd < 0) {				/* no input on fd is possible */
      _x11_redraw();
      return;
   }
   
   xfd = ConnectionNumber(display);
   readmask = (1 << fd) | (1 << xfd);
   ncheck = (xfd > fd ? xfd : fd) + 1;
   
   for(;;) {				/* wait for input to be ready */
      mask = readmask;
      if(select(ncheck,&mask,(int *)NULL,(int *)NULL,(void *)NULL) != 0) {
	 if(mask & (1 << xfd)) {
	    _x11_redraw();
	 }
	 if(mask & (1 << fd)) {
	    return;
	 }
      }
   }
#endif
}

static XEvent *
_x11_redraw()
{
   static int old_height = -1,		/* height is initialised to 0 */
   	      old_width;
   Pixmap new_backing;
   Region region;			/* Region to re-draw */
   static XEvent event;
   XRectangle rect;

   while(XCheckMaskEvent(display,event_mask,&event)) {
      switch (event.type) {
       case NoExpose:
       case GraphicsExpose:
	 break;
       case Expose:
	 if(have_backing) {		/* contents lost (bit_gravity
					   may not be implemented) */
	    continue;
	 }
	 region = XCreateRegion();
	 do {
	    rect.x = event.xexpose.x;
	    rect.y = event.xexpose.y;
	    rect.height = event.xexpose.height;
	    rect.width = event.xexpose.width;
	    XUnionRectWithRegion(&rect,region,region);
	 } while(XCheckTypedEvent(display,Expose,&event)) ;
	 XSetRegion(display,redraw_gc,region);
	 
	 XCopyArea(display,backing,wind,redraw_gc,0,0,width,height,0,0);
	 XDestroyRegion(region);
	 break;
       case ConfigureNotify:
	 size_x11();
	 if(old_height != height || old_width != width) {
	    if(!have_backing) {
	       new_backing = XCreatePixmap(display,wind,width,height,depth);
	       x_clear_pixmap(display,new_backing);
	       if(old_height >= 0) {
		  int dest_x,dest_y,
		  src_x,src_y;
		  unsigned int wwidth,hheight;
		  
		  if(width > old_width) {
		     wwidth = old_width;
		     src_x = 0;
		     dest_x = (width - old_width)/2;
		  } else {
		     wwidth = width;
		     src_x = (old_width - width)/2;
		     dest_x = 0;
		  }
		  if(height > old_height) {
		     hheight = old_height;
		     src_y = 0;
		     dest_y = (height - old_height)/2;
		  } else {
		     hheight = height;
		     src_y = (old_height - height)/2;
		     dest_y = 0;
		  }
		  XCopyArea(display,backing,new_backing,gc,src_x,src_y,
			    wwidth,hheight,dest_x,dest_y);
		  XFreePixmap(display,backing);
	       }
	       backing = new_backing;
	    }
	    old_height = height;
	    old_width = width;
	 }
	 break;
       case ButtonPress:		/* only in mask if in cursor mode */
       case KeyPress:
	 return(&event);
       default:
	 continue;
      }
   }
   XFlush(display);
   return(&event);
}

/****************************************************************/

void
x11_reloc(x,y)
int x,y;
{
   ;
}

/****************************************************************/

int
x11_set_ctype(colors,ncolors)
COLOR *colors;
int ncolors;
{
   return(-1);
}

/****************************************************************/

void
x11_ctype(r,g,b)
int r,g,b;
{
   XColor cdef;

   cdef.red = r << 8;
   cdef.green = g << 8;
   cdef.blue = b << 8;

   if(XAllocColor(display,cmap,&cdef) == 0) {
      msg_1d("Can't allocate colour (r,g,b) = (%d,",r);
      msg_1d("%d,",g);
      msg_1d("%d)\n",b);
   }
   x11_gflush();
   XSetForeground(display,gc,cdef.pixel);
}

/****************************************************************/
/*
 * Static utilities:
 */
static int
get_foreground(gc)
GC gc;
{
   XGCValues gcv;
   
   (void)XGetGCValues(display,gc,GCForeground,&gcv);
   return(gcv.foreground);
}

static int
X_get_opts(str,rdb)
char *str;
XrmDatabase *rdb;			/* merged database */
{
   int argc;
   char *argv[1 + NARG];		/* leave room for programme's name */
   char *ptr;
   XrmDatabase args_db = NULL,		/* resources from command line */
               xrdb_db = NULL;		/* and those managed by xrdb */
   
   argc = NARG;				/* max number of arguments */
   (void)str_to_argv(str,&argv[1],&argc);
   argv[0] = PROGNAME; argc++;
   XrmInitialize();
   
   XrmParseCommand(&args_db,options,NOPT,PROGNAME,&argc,argv);
   if(argc != 1) {
      int i;

      msg("Unknown arguments to DEVICE command:");
      for(i = 1;i < argc;i++) {
	 msg_1s(" %s",argv[i]);
      }
      msg("\n");
      usage();
   }

   if(*(ptr = get_resource(args_db,"preopened","")) != '\0') {
      if(str_scanf(ptr,"%d:%d",&display,&wind) != 2) {
	 wind = 0;
	 display = NULL;
	 msg("You need display:window with \"-p[reopened]\"\n");
      }
   }
   
   if(display == NULL) {
      ptr = get_resource(args_db,"display",(char *)NULL);
      if((display = XOpenDisplay(ptr)) == NULL) {
	 msg_1s("Could not open display \"%s\"\n",XDisplayName(ptr));
	 return(-1);
      }
   }
   
   if((ptr = XResourceManagerString(display)) != NULL) {
      xrdb_db = XrmGetStringDatabase(ptr);
      XrmMergeDatabases(xrdb_db,rdb);
   }
   XrmMergeDatabases(args_db,rdb);
      
   return(0);
}

static char *
get_resource(db,name,def_val)
XrmDatabase db;
char *name;
char *def_val;				/* default value */
{
   char fullname[50];
   char *option_type;			/* the type of the resource */
   static XrmValue value;

   sprintf(fullname,"%s.%s",PROGNAME,name);
   if(XrmGetResource(db,fullname,"",&option_type,&value) == True) {
      return(value.addr);
   } else {
      return(def_val);
   }
}

static void
usage()
{
   int i;

   msg("Legal options are:\n");
   msg("\t-help\n");
   for(i = 0;i < NOPT;i++) {
      msg_2s("\t%-40s (xrdb: %s)\n",options[i].option,options[i].specifier);
   }
}

static int
hand_XError(disp,event)
Display *disp;
XErrorEvent *event;
{
   char str[80];

   XGetErrorText(disp,event->error_code,str,80);
   msg_1s("X Error: %s\n",str);
   return(0);
}

/****************************************************************/

static int
hand_XIOError(disp)
Display *disp;
{
#ifdef lint
   *disp++;
#endif
   wind = 0;
   devnum = 0;
   set_dev();
   msg("Fatal X Error: closing device\n");
   hand_q(0);
   /*NOTREACHED*/
}

/****************************************************************/

static void
size_x11()
{
   int x,y;
   unsigned int b_width;
   Window root;

   XGetGeometry(display,wind,&root,&x,&y,&width,&height,&b_width,&depth);
   xscreen_to_pix = width/(float)SCREEN_SIZE;
   yscreen_to_pix = height/(float)SCREEN_SIZE;
   aspect = (float)(height)/width;
   ldef = 1/xscreen_to_pix;
}

/****************************************************************/

static void
x_clear_pixmap(disp,pixmap)
Display *disp;
Pixmap pixmap;
{
   XFillRectangle(disp,pixmap,erase_gc,0,0,width + 1,height + 1);
}
#endif
/*
 * Include a symbol for picky linkers
 */
#ifndef lint
static void linker_func() { linker_func(); }
#endif

