#include "copyright.h"
/*********************************************************************/
/*       AXIS.C - draw box with axis                                 */
/*-------------------------------------------------------------------*/
/*       box()                                                       */
/*       axis()                                                      */
/*       ticksize()						     */
/*       format()						     */
/*       grid()							     */
/*       notation()						     */
/*								     */
/*********************************************************************/
#include "options.h"
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "vectors.h"
#include "defs.h"
#include "sm.h"
#include "sm_declare.h"

#define INT(X) (int)((X) + 0.5)
#define LABEL_OFFSET 400		/* distance of numbers from axis,
					   and axis label from numbers */

static char x_num_fmt[20]="",
	    y_num_fmt[20]="";
static float xsmall = 0.,
	     ysmall = 0.,
	     xbig   = 0.,
	     ybig   = 0.;
extern int sm_interrupt,			/* respond to ^C */
           sm_verbose;			/* how chatty should I be? */	    
static int check_ti(),
	   x_numb_min = SCREEN_SIZE,	/* height + depth of x-axis numbers */
	   xlexp = -4,
           xhexp =  4,
	   y_numb_min = SCREEN_SIZE,	/* length of y-axis numbers */
           ylexp = -4,
           yhexp =  4,
	   TeX = -1;			/* Use TeX strings? */
static char *user_label = NULL;		/* user-supplied label for big ticks */
static void axistick(),
	    format_string(),
	    gridfn(),
  	    tick(),
	    ticklabel();
static void vec_axistick();

void
sm_box(labelxl, labelyl, labelxu, labelyr)
int labelxl, labelyl, labelxu, labelyr;
{
   float asave = aangle,
   	 ninety = 90.0,
	 zero = 0.0;
   int lsave = lltype;

   if(lltype != -10) {			/* not erase */
      sm_ltype(0);
   }
   set_angle(&zero,1);
   
   x_numb_min = y_numb_min = SCREEN_SIZE;
   
   (void)sm_axis(fx1,fx2,xsmall,xbig,gx1,gy1,gx2-gx1,labelxl,0);
   (void)sm_axis(fx1,fx2,xsmall,xbig,gx1,gy2,gx2-gx1,labelxu,1);
   set_angle(&ninety,1);
   (void)sm_axis(fy1,fy2,ysmall,ybig,gx1,gy1,gy2-gy1,labelyl,1);
   (void)sm_axis(fy1,fy2,ysmall,ybig,gx2,gy1,gy2-gy1,labelyr,0);
   
   set_angle(&asave,1);
   sm_ltype(lsave);
}

void
sm_grid(line_freq,dirs)
int line_freq;  /* 0 for grid lines only at labelled ticks, 1 for all ticks */
int dirs;				/* which lines: 0: x&y, 1: x 2: y */
{
   float a1, a2,
   	 grid_par[3],			/* used by axistick (q.v.) */
   	 ninety = 90.0,
   	 old_angle,
   	 zero = 0.0;

   old_angle = aangle;
   grid_par[1] = line_freq;
/*
 * x axis
 */
   if(dirs == 0 || dirs & 01) {
      set_angle(&zero,1);
      if(xsmall < 0) {
	 a1 =  pow(10.0, fx1);
	 a2 =  pow(10.0, fx2);
      } else {
	 a1 = fx1;
	 a2 = fx2;
      }
      grid_par[ 0 ] = gy2 - gy1;
      grid_par[ 2 ] = 0;
      axistick( a1, a2, xsmall, xbig, gx1, gy1, gx2-gx1, grid_par, gridfn );
   }
/*
 * y axis
 */
   if(dirs == 0 || dirs & 02) {
      set_angle(&ninety,1);
      if(ysmall < 0) {
	 a1 = pow(10.0, fy1);
	 a2 =  pow(10.0, fy2);
      } else {
	 a1 = fy1;
	 a2 = fy2;
      }
      grid_par[ 0 ] = gx2 - gx1;
      grid_par[ 2 ] = 1;
      axistick( a1, a2, ysmall, ybig, gx1, gy1, gy2-gy1, grid_par, gridfn );
      set_angle(&old_angle,1);
   }
}

/*
 * set axis label formats
 */
void
sm_format( x_format, y_format )
char *x_format, *y_format;
{
   if(!strcmp(x_format,"1")) {
       x_num_fmt[0] = '\0';
   } else if(strcmp(x_format,"0") != 0) {
       format_string( x_format, x_num_fmt);
   }
   if(!strcmp(y_format,"1")) {
       y_num_fmt[0] = '\0';
   } else if(strcmp(y_format,"0") != 0) {
       format_string( y_format, y_num_fmt);
   }
}

void
sm_ticksize(asx,abx,asy,aby)
double asx,abx,asy,aby;
{
   if(check_ti(asx,abx) == 0) {
      xsmall = asx; xbig = abx;
   }
   if(check_ti(asy,aby) == 0) {
      ysmall = asy; ybig = aby;
   }
}

static int
check_ti(small,big)
float small,big;
{   
   if(small > 0.0) {
      if(small > big) {
	 msg("Small tick spacing must be <= big spacing\n");
	 return(-1);
      }
   } else if(small == 0.0) {
      if(big != 0.0) {
	 msg("You cannot specify the big tick spacing but not the small\n");
	 return(-1);
      }
   } else {
      if(big != 0.0 && (-small) > fabs(big)) {
	 msg("Small tick spacing must be <= big spacing\n");
	 return(-1);
      }
   }
   return(0);
}


/*---------------------------------------------------------------------------*/
/*	Makes a labeled axis from A1 to A2 at (AX,AY), length ALEN.	     */
/*	  If ABig > 0, use it for spacing of large ticks		     */
/*	  If ASmall <0, make logarithmic axis, if =0, the default.	     */
/*	  If ASmall >0, try to use it for the small ticks.		     */
/*	  ILABEL = 0 for no labels, 1 for parallel, 2 for perpend.	     */
/*        ICLOCK%2 = 0 for counterclockwise, 1 for clockwise ticks	     */
/*        ICLOCK/2 = 0 for orthogonal, 1 for vertical, 2 for horizontal ticks*/
/*---------------------------------------------------------------------------*/
int
sm_axis(a1,a2,as,ab,ax,ay,alen,ilabel,iclock)
double a1,a2,			/* labels from a1 to a2  */
       as,ab; 			/* if as < 0, logarithmic */
int ax,ay,alen,
    ilabel,iclock;
{
   float par[3];
   int ax2,ay2;
   float xasp,yasp;
   
   if(sm_interrupt) return(-1);

   TeX = (*print_var("TeX_strings") != '\0'); /* do we want TeX strings? */
   
   if(as < 0) {				/* logarithmic */
      if(fabs(a1) > MAX_POW || fabs(a2) > MAX_POW) {
	 msg("|limits| are too large for logarithmic axes\n");
	 as = 0;
      } else {
	 a1 = pow(10.,a1);
	 a2 = pow(10.,a2);
      }
   }

   if(aspect > 1.0) {
      xasp = aspect;
      yasp = 1.0;
   } else {
      xasp = 1.0;
      yasp = aspect;
   }
   alen /= sqrt(pow(sm_cos*xasp,2.0) + pow(sm_sin/yasp,2.0));
   ax2 = ax + alen*sm_cos*xasp + 0.5;
   ay2 = ay + alen*sm_sin/yasp + 0.5;
   sm_grelocate((float)ax,(float)ay);
   gdraw((float)ax2,(float)ay2);

   if(iclock < 0 || iclock > 6) {
      msg_1d("Invalid value %d of iclock in axis; setting to 0\n",iclock);
      iclock = 0;
   }
   par[0] = iclock;
   iclock = iclock%2;			/* labels must be orthogonal */
   if(ilabel == 3) return(0);		/* don't mark ticks on axis */
   axistick(a1,a2,as,ab,ax,ay,alen,par,tick);
   if(ilabel == 0) return(0);		/* don't label axis */
   if(ilabel == 1) {
      par[0] = 1;  par[1] = 0;
   } else {
      par[0] = 2 * iclock; par[1] = 1;
   }
   par[2] = iclock;
   axistick(a1,a2,as,ab,ax,ay,alen,par,ticklabel);

   return(0);
}

static void
axistick(a1,a2,as,ab,ax,ay,alen,par,tickfn)
float a1,a2,as,ab;
int ax,ay,alen;
float *par;
void (*tickfn)();
{
   double step, tick_place, value;
   float decade = 0,			/* initialise to 0 to placate
					   flow-analysing compilers */
   	 length,
    	 xpos, ypos,
   	 adiff, diff,
   	 mantissa,
   	 oldang;
   int big = 0,				/* force a big tick, initialised to
					   placate flow-analysing compilers */
       exp_val, offset,
       i,
       nchars, nlabel, ticks, smticks;
   float xasp,yasp;
   
   if(a1 == a2) return;
/*
 *  work out the placement of the ticks for the case where the user
 *  lets SM decide the big tick spacing
 */
   if(as >= 0 ) {
      if(ab == 0) {
	 float zerop = 0.001;		/* zero + delta */
	 smticks = 5;
	 nchars = 10;
/*
 * now work out how many labels can fit on the axis, depending on
 * axis length,  character size, number of characters allowed
 */
	 oldang = aangle;
	 set_angle(&zerop,1);		/* force software chars for cheight */
	 nlabel = (alen/(nchars*cheight)) + 0.5;
	 set_angle(&oldang,1);
	 if(nlabel < 4) {
	    if(sm_verbose > 1) {
	       msg_1d("I want only %d labels\n",nlabel);
	    }
	    nlabel = 4;			/* got to have at least 4 values */
	 }
/*
 *  now work out the numeric range in the labels
 */
	 adiff = fabs( a2 - a1 );
	 diff = log10( adiff/ nlabel );
	 exp_val = diff;
	 if(diff < 0) exp_val = exp_val - 1;
	 mantissa = diff - exp_val;
	 
	 if(mantissa < .15 ) {
	    ticks = 1;
	 } else if ( mantissa < .5 ) {
	    ticks = 2;
	    smticks = 4;
	 } else if (mantissa < .85 ) {
	    ticks = 5;
	 } else {
	    ticks = 10;
	 }
/*
 * now work out the values at which each tick mark should be placed
 */
	 step = ticks*pow( 10.0, (double)(exp_val) );
	 
	 if(as > 0) {	/* The user specified only the SMALL tick spacing */
	    smticks = step/as + 0.5;
	 }
      } else if(ab > 0) {	/* user chose big and small ticks */
	 if(as == 0) {
	    as = ab/5;
	 }
	 step = ab;
	 smticks = step/as + 0.5;
      } else {				/* ab < 0 */
	 msg("You can't have big spacing < 0 with non-logarithmic axes\n");
	 return;
      }
   } else {				/* a logarithmic axis is specified  */
      if (a1 <= 0. || a2 <= 0.) 
	return;
      a1 = log10( a1 );
      a2 = log10( a2 );
      step = (ab == 0.0) ? 10 : (ab > 0.0 ? ab : -ab);
      smticks = step/(-as) + 0.5;
   }
   
   if(a2 < a1) step = -step;
   if(as >= 0) {
      offset = (int)(a1/step);
      if(a1/step < 0) offset--;
      tick_place = step*offset;
   } else {
      if(a2 > a1) {
	 offset = (int)a1;
      } else {
	 offset = (int)ceil(a1);
	 step /= 10.;
      }
      if(a1 < 0) offset--;
      decade = pow(10.0,(float)offset);
      if(a2 > a1) {
	 tick_place = 0.0;
      } else {
	 tick_place = 1.0;
      }
   }
/*
 * these are the loops for making the tick marks; the while loop for the big
 * ticks and the for loop for the small ticks 
 */
   a1 -= 1.0e-6*(a2 - a1);	/* allow for roundoff */
   a2 += 1.0e-6*(a2 - a1);

   if(aspect > 1.0) {
      xasp = aspect;
      yasp = 1.0;
   } else {
      xasp = 1.0;
      yasp = aspect;
   }

   while(!sm_interrupt) {
      for(i = 0;i < smticks;i++) {
	 value = tick_place + step*i/smticks;
	 if (fabs(value) < fabs(step * 1.0e-12)) {
	    value = 0.0;
	 }
	 if(as >= 0) {
	    length = alen * (value - a1)/(a2 - a1);
	 } else {
	    if(ab >= 0.0 && value > 10.0) break;
	    if(value <= 0.0) {
	       if(smticks > 1) {	/* we'll never get anywhere if i==1 */
		  big = 1;		/* force next tick to be big */
		  continue;
	       } else {
		  value = 1;
	       }
	    } else if(value < 0.1) {
	       continue;
	    }
	    value *= decade;
	    length = alen * (log10(value) - a1)/(a2 - a1);
	 }
	 if(length < -1) {
	    big = 0;			/* don't need a big tick */
	    continue;
	 } else if(length > alen) {
	    return;
	 }
	 xpos = length*sm_cos*xasp + ax + 0.5;
	 ypos = length*sm_sin/yasp + ay + 0.5;
	 (*tickfn)(xpos,ypos,value,(i == 0 || big),par);
	 big = 0;
      }
      tick_place += step;
      if(as < 0.0) {
	 if(a2 > a1) {
	    if(tick_place >= 9.9999) {
	       tick_place = 0.0;
	       decade *= 10;
	       if(ab < 0) step /= 10;
	    }
	 } else {
	    if(tick_place < 0.1) {
	       tick_place = 1.0;
	       decade /= 10;
	       if(ab < 0) step *= 10;
	    }
	 }
      }
   }
}

/*****************************************************************************/
/*
 * Like axis, but instead of specifying as and ab specify two vectors,
 * giving the big and small ticks
 */
void
vec_axis(a1,a2,small,big,lab,ax,ay,alen,ilabel,iclock)
double a1,a2;
VECTOR *small,*big;
VECTOR *lab;
int ax,ay,alen,
    ilabel,iclock;
{
   float par[3];
   int ax2,ay2;
   float xasp,yasp;
   
   if(sm_interrupt) return;

   TeX = (*print_var("TeX_strings") != '\0'); /* do we want TeX strings? */
   
   if(aspect > 1.0) {
      xasp = aspect;
      yasp = 1.0;
   } else {
      xasp = 1.0;
      yasp = aspect;
   }
   alen /= sqrt(pow(sm_cos*xasp,2.0) + pow(sm_sin/yasp,2.0));
   ax2 = ax + alen*sm_cos*xasp + 0.5;
   ay2 = ay + alen*sm_sin/yasp + 0.5;
   sm_grelocate((float)ax,(float)ay);
   gdraw((float)ax2,(float)ay2);

   par[0] = iclock;
   iclock = iclock%2;			/* labels must be orthogonal */
   if(ilabel == 3) return;		/* don't mark ticks on axis */
   vec_axistick(a1,a2,big,1,ax,ay,alen,par,tick,(VECTOR *)NULL);
   vec_axistick(a1,a2,small,0,ax,ay,alen,par,tick,(VECTOR *)NULL);
   if(ilabel == 0) return;		/* don't label axis */
   if(ilabel == 1) {
      par[0] = 1;  par[1] = 0;
   } else {
      par[0] = 2 * iclock; par[1] = 1;
   }
   par[2] = iclock;
   vec_axistick(a1,a2,big,1,ax,ay,alen,par,ticklabel,lab);
   vec_axistick(a1,a2,small,0,ax,ay,alen,par,ticklabel,(VECTOR *)NULL);
}

static void
vec_axistick(a1,a2,vec,big,ax,ay,alen,par,tickfn,vec_lab)
float a1,a2;
VECTOR *vec;				/* vector of tick marks */
int big;				/* are they big ticks? */
int ax,ay,alen;
float *par;
void (*tickfn)();
VECTOR *vec_lab;			/* vector of labels to draw */
{
   double value;
   float length,
    	 xpos, ypos;
   int i;
   float xasp,yasp;
   
   if(aspect > 1.0) {
      xasp = aspect;
      yasp = 1.0;
   } else {
      xasp = 1.0;
      yasp = aspect;
   }
   for(i = 0;i < vec->dimen;i++) {
      value = vec->vec.f[i];
      if(value < a1 || value > a2) continue;
      length = alen*(value - a1)/(a2 - a1);
      xpos = length*sm_cos*xasp + ax;
      ypos = length*sm_sin/yasp + ay;
      if(vec_lab == NULL) {
	 user_label = NULL;
      } else {
	 user_label = (vec_lab->dimen == 1) ?
				     vec_lab->vec.s.s[0] : vec_lab->vec.s.s[i];
      }
      (*tickfn)(xpos,ypos,value,big,par);
   }
   user_label = NULL;
}

/*****************************************************************************/
/*
 * Draw the axis labels
 */
void
sm_xlabel( str )
char *str;
{
   float slength,
   	 sdepth,sheight,
	 xstart,ystart;
   int lsave;
   
   lsave = lltype;
   if(lltype != -10) sm_ltype( 0 );
   string_size(str,&slength,&sdepth,&sheight);
   xstart = gx1 + (gx2 - gx1 - slength)/2.0;

   if(y_numb_min == SCREEN_SIZE) {	/* no idea where axis labels are */
      ystart =  gy1 - 1.5*cheight;
   } else {
      ystart = y_numb_min;
   }
   ystart -= sheight + eexpand*LABEL_OFFSET;
   if(ystart < 100 + sdepth) {
      if(sm_verbose) {
	 int dy  = (100 + sdepth) - ystart;
	 dy = 500*(int)(dy/500 + 0.999);
	 if(dy > 0) {
	    msg("The x-axis label will probably overlap the tick numbers;\n");
	    msg_1d("you may want to increase the starting y LOCATION by %d\n",
		   dy);
	 }
      }
      ystart = 100 + sdepth;
   }

   sm_grelocate(xstart,ystart);
   draw_string(str);
   sm_ltype( lsave );
}

void
sm_ylabel( str )
char *str;
{
   float angsave,
   	 ninety = 90.0,
     	 slength,
   	 sdepth,sheight,
     	 xstart, ystart;
   int lsave;
   
   angsave = aangle;
   lsave = lltype;
   if(lltype != -10) sm_ltype( 0 );
   if(aangle == 0.0) {
      set_angle(&ninety,1);
   }
   string_size(str,&slength,&sdepth,&sheight);
   
   if(x_numb_min == SCREEN_SIZE) {	/* no idea where axis labels are */
      xstart =  gx1 - 4*cwidth - sdepth;
   } else {
      xstart = x_numb_min;
   }
   xstart -= sdepth + eexpand*LABEL_OFFSET;
   if(xstart < 100 + sheight) {
      if(sm_verbose) {
	 int dx  = (100 + sheight) - xstart;
	 dx = 500*(int)(dx/500 + 0.999);
	 if(dx > 0) {
	    msg("The y-axis label will probably overlap the tick numbers;\n");
	    msg_1d("you may want to increase the starting x LOCATION by %d\n",
		   dx);
	 }
      }
      xstart = 100 + sheight;
   }

   ystart = gy1 + (gy2 - gy1 - slength)/2.0;
   sm_grelocate(xstart,ystart);
   draw_string(str);
   set_angle(&angsave,1);
   sm_ltype(lsave);
}

/*****************************************************************************/
/*
 * Invalidate the saved label positions
 */
void
reset_label_pos()
{
   x_numb_min = y_numb_min = SCREEN_SIZE;
}

/*************************************************************/
/*
 * draw the tick marks at position xloc, yloc
 */
static void
tick(xloc,yloc,value,tsize,par)
float xloc, yloc,
      value;
int tsize;
float *par;
{
   int size;			/* size of tickmark */
   float xtick, ytick;		/* end of tick */
   float xasp,yasp;		/* aspect ratios */
   float tick_sin, tick_cos;	/* virtual axis direction for ticks*/
   int iclock;			/* an integer for ICLOCK */
   
   iclock = INT(par[0]);
   if(iclock/2 == 0) {			/* orthogonal tick */
      tick_sin = sm_sin;				
      tick_cos = sm_cos;
   } else if(iclock/2 == 1) {		/* vertical tick */
      tick_sin = 0;
      tick_cos = (sm_cos>0)? 1 : -1;
   } else {				/* horizontal tick */
      tick_sin = (sm_sin>0)? 1 : -1;
      tick_cos = 0;
   }
   if(aspect > 1.0) {
      xasp = aspect;
      yasp = 1.0;
   } else {
      xasp = 1.0;
      yasp = aspect;
   }

   size = (cheight/1.5)/sqrt(pow(tick_sin*xasp,2.0) + pow(tick_cos/yasp,2.0));
   if(iclock%2 == 1 ) {			/* clockwise tick */
      size = -size;
   }
   if(tsize == 0) {
      size /= 2;
   }

   xtick = xloc - size*tick_sin*xasp;
   ytick = yloc + size*tick_cos/yasp;

   sm_grelocate(xloc,yloc);
   gdraw(xtick,ytick);
}

/*
 *  put on the tick labels  at location xloc, yloc  
 *  par[0] is 0, 1, 2, for left, center, right justified
 *  par[1] is 0 for labels parallel to the axis, 1 for perpendicular
 *  par[2] is 0 for clockwise labels on the axis, 1 for anti-clockwise
 */

static void
ticklabel(xloc,yloc,value,tsize,par)
float xloc, yloc,
      value;
int tsize;
float *par;
{
    static char string[41];
    float ang,
    	  dx, dy, xstart, ystart,
	  sheight,slength;
    float xasp,yasp;

    if(tsize == 0) {
       return;
    }
    if(user_label != NULL) {
       strcpy(string,user_label);
    } else {
       num_to_ascii(value,string);
    }
    
    if(aspect > 1.0) {
       xasp = aspect;
       yasp = 1.0;
    } else {
       xasp = 1.0;
       yasp = aspect;
    }
    
    if ((int)par[1] == 0) {		/* parallel to axis */
       string_size(string,&slength,(float *)NULL,&sheight);
       dx = -par[0]*slength/2;
       if(par[2] < 0.5) {		/* clockwise */
	  dy = -(eexpand*LABEL_OFFSET + sheight);
       } else {				/* widdershins */
	  dy = eexpand*LABEL_OFFSET;
       }
    
       dx /= sqrt(pow(sm_cos*xasp,2.0) + pow(sm_sin/yasp,2.0));
       dy /= sqrt(pow(sm_sin*xasp,2.0) + pow(sm_cos/yasp,2.0));
       xstart = (dx*sm_cos - dy*sm_sin)*xasp + xloc;
       ystart = (dx*sm_sin + dy*sm_cos)/yasp + yloc;
       sm_grelocate(xstart,ystart);
       draw_string(string);
    } else {				/* perpendicular to axis */
       ang = aangle - 90;
       set_angle(&ang,1);
       string_size(string,&slength,(float *)NULL,&sheight);
       dx = -par[0]*slength/2;
       if(par[2] < 0.5) {		/* clockwise */
	  dx += eexpand*LABEL_OFFSET;
       } else {				/* widdershins */
	  dx += -eexpand*LABEL_OFFSET;
       }
       dy = -sheight/2;
    
       dx /= sqrt(pow(sm_cos*xasp,2.0) + pow(sm_sin/yasp,2.0));
       dy /= sqrt(pow(sm_sin*xasp,2.0) + pow(sm_cos/yasp,2.0));
       xstart = (dx*sm_cos - dy*sm_sin)*xasp + xloc;
       ystart = (dx*sm_sin + dy*sm_cos)/yasp + yloc;
       sm_grelocate(xstart,ystart);
       draw_string(string);
       ang = aangle + 90;
       set_angle(&ang,1);
    }

    if(x_numb_min > xstart && xstart < gx1) x_numb_min = xstart;
    if(y_numb_min > ystart && ystart < gy1) y_numb_min = ystart;
}

static void
gridfn( x, y, value, tsize, par)
float x, y, value;
int tsize;
float *par;
{
   float xend, yend;
   
   if(tsize == 0 && par[1] != 1) return;
   
   if(par[ 2 ] > 0.5)  {
      xend = x + par[ 0 ];
      yend = y;
   } else {
      xend = x;
      yend = y + par[ 0 ];
   }
   draw_dline( INT(x), INT(y), INT(xend), INT(yend) );
}

/******************************************************/
/*
 * convert a number to a string for plotting
 *
 *  value    = the value to be printed on the plot (input)
 *  string = string to be printed on the plot (output)
 */
#define NDIGIT 6			/* maximum number of digits */

void
num_to_ascii(value,string)
double value;
char *string;
{
   char *char_value,
	*s;
   int decpt,				/* position of decimal point */
       exponent = 0,			/* exponential notation? */
       lexp,hexp,			/* lowest and highest exponents */
       sign;
   
   if(aangle <= 45.) {     /*   x axis   */
      lexp = xlexp;
      hexp = xhexp;
      if (x_num_fmt[0] != '\0' ) {	/* user supplied the x-axis format */
         (void)sprintf(string,x_num_fmt,value);
         return;
      } 
   } else {
      lexp = ylexp;
      hexp = yhexp;
      if(y_num_fmt[0] != '\0' ) {	/* the user supplied y-axis format */
         (void)sprintf( string, y_num_fmt,value );
         return;
      }
   }
   if(TeX < 0) {
      TeX = (*print_var("TeX_strings") != '\0'); /* do we want TeX strings? */
   }
/*
 * OK, now deal with the number ourelves
 */
   char_value = ecvt(value,NDIGIT,&decpt,&sign);
   for(s = char_value + NDIGIT - 1;*s == '0';s--) *s = '\0';
   
   s = string;
   if(*char_value == '\0') {
      sprintf(s,"0");
      return;
   }
   if(decpt <= lexp || decpt > hexp) {
      exponent = decpt - 1;
      decpt = 1;
   }
   if(sign) *s++ = '-';
   if(decpt > 0) {
      while(decpt-- != 0) {
	 *s++ = (*char_value == '\0' ? '0' : *char_value++);
      }
      if(*char_value != '\0') *s++ = '.';
      while(*char_value != '\0') *s++ = *char_value++;
   } else {
      sprintf(s,"0."); s += 2;
      while(decpt++ != 0) *s++ = '0';
      while(*char_value != '\0') *s++ = *char_value++;
   }
   *s = '\0';
   if(exponent == 0) {
      return;
   }
/*
 * Now the exponential part
 */
   if(string[0] == '1' && string[1] == '\0') {
      s = string;			/* string is "1" from string */
   } else {
      (void)sprintf(s,TeX ? "\\times " : "\\g*");
      s += TeX ? 7 : 3;
   }
   (void)sprintf(s,TeX ? "10^{" : "10\\\\u");
   s += TeX ? 4 : 5;
/*
 * now line the exponents up prettily (i.e. if exponent between 0 and 9,
 * put a space between the 10 and the exponent, else not) if $line_up_exponents
 * is not defined
 */
   if(exponent < 0 || exponent > 9 || *print_var("line_up_exponents") == '\0'){
      (void)sprintf(s,"%d",exponent);
   } else {
      (void)sprintf(s," %d",exponent);
   }
   if(TeX) {
      while(*s != '\0') s++;
      sprintf(s,"}");
   }
   return;
}

/************************************************/
/*
 * Given a Fortran format string, produce a C one. If the format
 * includes a %, it is passed through unscathed.
 */
static void
format_string(in,out)
char *in,			/* input format */
     *out;			/* output format */
{
   char c;
   int i,j,n;

   (void)str_scanf(in,"%c",&c);
   if(c == 'd' || c == 'D' || c == 'E') c = 'e';
   else if(c == 'F') c = 'f';
   else if(c == 'G') c = 'g';
   
   switch (c) {
   case '\0':
      return;
   case '%':
      (void)sprintf(out,"%s",in);
      return;
   case 'e':
   case 'f':
   case 'g':
      if((n = str_scanf(in,"%*c%d.%d",&i,&j)) <= 0) {	/* no width field */
	 (void)sprintf(out,"%%%c",c);
      } else if(n == 1) {
	 (void)sprintf(out,"%%%d%c",i,c);
      } else {
	 (void)sprintf(out,"%%%d.%d%c",i,j,c);
      }
      break;
   case 'i':
   case 'I':
      if(str_scanf(in,"%*c%d.%d",&i) <= 0) {	/* no width field */
	 (void)sprintf(out,"%%g");
      } else {
	 (void)sprintf(out,"%%%dg",i);
      }
      break;
   default:
      msg_1s("Illegal format %s\n",in);
      (void)sprintf(out,"%%g");
      break;

   }
}

void
sm_notation(xlo, xhi, ylo, yhi)
double xlo, xhi, ylo, yhi;
{
   xlexp = xlo;
   xhexp = xhi;
   ylexp = ylo;
   yhexp = yhi;
}
