#include "copyright.h"
#include <stdio.h>
#include "options.h"
#include "sm.h"
#include "defs.h"
#include "sm_declare.h"
  
extern int sm_verbose;

int
sm_conn(xarray, yarray, npts)
float xarray[], yarray[];
int npts;
{
   int i;
   float epsilon = 1.0;
   int nfan,*karray;
   
   if(npts < 2) return(-1);

   if(*print_var("fan_compress") == '\0') { /* not defined */
      sm_relocate(xarray[0], yarray[0]);
      for(i = 1;i < npts;i++) {
         sm_draw(xarray[i],yarray[i]);
      }
   } else {				/* fan compress data */
      if((karray = (int *)malloc((unsigned)npts*sizeof(int))) == NULL) {
	 msg("Can't allocate fan compression array\n");
	 return(-1);
      }
      fan_compress(xarray, yarray, npts, epsilon, karray, &nfan);
      
      if(sm_verbose) {
	 msg_1d("Fan Compression: Plotting %d of ",nfan);
	 msg_1d("%d points ",npts); 
	 msg_1f("with epsilon = %f\n",epsilon);
      }
      
      sm_relocate(xarray[karray[0]],yarray[karray[0]]);
      for(i = 1;i < nfan;i++) {
	 sm_draw(xarray[karray[i]], yarray[karray[i]]);
      }
      free((char *)karray);
   }
   return(0);
}

/********************************************************/
/*
 * Connect points if larray is true; only connect continuously true segments
 */
int
conn_if(xarray, yarray, larray, npts)
float xarray[], yarray[],
      larray[];
int npts;
{
    int connecting = 0,			/* are we currently connecting? */
        i;
    
    if(npts < 2) return(-1);
    for(i = 0;i < npts;i++) {
       if(!larray[i]) {
	  connecting = 0;
	  continue;
       }
       
       if(!connecting) {
	  sm_relocate(xarray[i], yarray[i]);
	  connecting = 1;
       } else {
	  sm_draw(xarray[i], yarray[i]);
       }
    }
    return(0);
}

/********************************************************/

int
sm_histogram(xarray, yarray, npts)
float xarray[], yarray[];
int npts;
{
   int i ;
   float xav;
   
   if(npts < 2) return(-1);
   sm_relocate(0.5*(3*xarray[0] - xarray[1]), yarray[0]);
   xav = 0.5*(xarray[1] + xarray[0] );
   sm_draw(xav, yarray[0]);
   for (i=1; i<npts; i++ ) {
      xav = 0.5*(xarray[i] + xarray[i-1]);
      sm_draw(xav, yarray[i-1]);
      sm_draw(xav, yarray[i]);
   }
   sm_draw(0.5*(3*xarray[npts-1] - xarray[npts-2]), yarray[npts-1]);
   
   return(0);
}

/********************************************************/
/*
 * Draw a histogram, dropping points where the logical array is false
 */
int
histogram_if(xarray, yarray, larray, npts)
float xarray[], yarray[],
      larray[];
int npts;

{
   int connecting = 0,
       i;
   float xav;
   
   if(npts < 2) return(-1);
   
   for(i = 0;i < npts;i++) {
      if(i == 0) {
	 xav = 0.5*(3*xarray[0] - xarray[1]);
      } else {
	 xav = 0.5*(xarray[i] + xarray[i-1]);
      }
      if(!larray[i]) {
	 if(connecting) {
	    sm_draw(xav, yarray[i-1]);
	 }
	 connecting = 0;
	 continue;
      }
      
      if(!connecting) {
	 connecting = 1;
	 sm_relocate(xav,yarray[i]);
      } else {
	 sm_draw(xav,yarray[i-1]);
      }
      sm_draw(xav, yarray[i]);
   }
   if(connecting) {
      sm_draw(0.5*(3*xarray[npts-1] - xarray[npts-2]), yarray[npts-1]);
   }
   
   return(0);
}

/********************************************************/

int
sm_errorbar( xarray, yarray, earray, loc, npts)
float xarray[], yarray[], earray[];
int loc, npts;
{
   float angsav,
   	 ninety = 90.0,
	 xx, yy,
   	 zero = 0.0;
   int j;

   if (npts < 1) return(-1);
   angsav = aangle;
   if(loc == 1 || loc == 3) {
       set_angle(&ninety,1);
   } else {
       set_angle(&zero,1);
   }
   for (j=0; j<npts; j++) {
       sm_relocate(xarray[j],yarray[j]);
       switch(loc) {
	case 1:
           xx = xarray[j] + earray[j];
           yy = yarray[j];
	   break;
	 case 2:
           xx = xarray[j];
           yy = yarray[j] + earray[j];
	   break;
	 case 3:
           xx = xarray[j] - earray[j];
           yy = yarray[j];
	   break;
	 case 4:
           xx = xarray[j];
           yy = yarray[j] - earray[j];
	   break;
	 default:
           xx = xarray[j] + earray[j];
           yy = yarray[j];
	   msg_1d("Illegal direction for error bar %d\n",loc);
	   break;
       }
       sm_draw( xx, yy );
       sm_draw_point(xx,yy,2,0);
   }
   set_angle(&angsav,1);
   return(0);
}



