#include "copyright.h"
#include <stdio.h>
#include "options.h"
#include "sm_declare.h"
/*
 * Shade the area inside a curve. shade_hist() will first convert the
 * curve into a histogram, and then shade between that histogram and the
 * x axis.
 */
#include <stdio.h>
#include <math.h>
#include "sm.h"

#define NCROSS 40			/* maximum crossings per shade line */
  
extern int sm_interrupt,			/* respond to ^C */
  	   sm_verbose;			/* Be chatty? */
static float cosa,sina;			/* cos and sin of desired rotation */
static void draw_seg(),
	     rotate();

void
sm_shade(delta,a,b,npoint)
int delta;				/* spacing between lines */
float *a,*b;				/* (a,b) is curve defining region */
int npoint;				/* dimension of a,b */
{
   float alpha,			/* angle to rotate by */
   	 crossing[NCROSS],		/* where line segments cross curve */
   	 cx[4],cy[4],			/* corners of box */
   	 dy,				/* line spacing in y units */
   	 y;				/* current (rotated) y value */
   int i,i1,
       j;				/* index into crossing[] */

   if(delta < ldef) {
      delta = ldef;			/* no point drawing them closer */
   }
/*
 * Figure out angle that'll give aangle when drawn
 */
   if(sm_cos == 0.0) {
      cosa = 0;
      sina = sm_sin/fabs(sm_sin);
   } else if(sm_sin == 0.0) {
      cosa = sm_cos/fabs(sm_cos);
      sina = 0;
   } else {
      alpha = atan((sm_sin/fsy)/(sm_cos/fsx));
      sina = fabs(sin(alpha)/sm_sin)*sm_sin;
      cosa = fabs(cos(alpha)/sm_cos)*sm_cos;
   }
   dy = delta*hypot(sina/fsx,cosa/fsy);
   if(dy == 0) dy = 1;
   
   cx[0] = fx1; cy[0] = fy1;
   cx[1] = fx2; cy[1] = fy1;
   cx[2] = fx1; cy[2] = fy2;
   cx[3] = fx2; cy[3] = fy2;
   rotate(a,b,npoint);
   rotate(cx,cy,4);

   sort_flt(cx,4);			/* find highest and lowest corners */
   sort_flt(cy,4);
/*
 * Now we can draw lines
 */
   for(y = cy[0];!sm_interrupt && y < cy[3];y += dy) {
      for(i = j = 0;i < npoint;i++) {
	 i1 = (i == npoint - 1) ? 0 : i + 1; /* wrap start onto end */
	 if(i1 != 0 && (b[i1] - y) == 0.0) {
	    continue;			/* don't find it twice */
	 }
	 if((b[i] - y)*(b[i1] - y) <= 0) { /* a crossing */
	    if(y - b[i] == 0.0) {
	       crossing[j++] = a[i];	/* avoid division by zero */
	    } else {
	       crossing[j++] = a[i] + (a[i1] - a[i])/(b[i1] - b[i])*(y - b[i]);
	    }
	    if(j >= NCROSS) {
	       if(sm_verbose) {
		  msg_1d("Only %d boundary crossings per line in shade\n",
			 					NCROSS);
	       }
	       break;
	    }
	 }
      }
      if(j > 0) {			/* lines to draw */
	 sort_flt(crossing,j);
	 for(i = 0;i < j - 1;i += 2) {
	    draw_seg(crossing[i],crossing[i+1],y);
	 }
	 if(j%2 == 1) {			/* an incomplete line */
	    draw_seg(crossing[j - 1],cx[3],y);
	 }
      }
   }
}

/*****************************************************************/
/*
 * Shade a curve as a histogram
 */
int
shade_hist(delta,a,b,npoint)
int delta;				/* spacing between lines */
float *a,*b;				/* (a,b) is curve defining region */
int npoint;				/* dimension of a,b */
{
   float xav,*x,*y;
   int i;
/*
 * Need space for histogram `corners'; notice that we need 2N + 2 not 2N
 */
   if((x = (float *)malloc(2*(npoint + 1)*sizeof(float))) == NULL) {
      msg("Can't allocate storage for x\n");
      return(-1);
   }
   if((y = (float *)malloc(2*(npoint + 1)*sizeof(float))) == NULL) {
      msg("Can't allocate storage for y\n");
      free((char *)x);
      return(-1);
   }
/*
 * Convert curve to histogram, and anchor ends on x axis
 */
   x[0] = x[1] = 0.5*(3*a[0] - a[1]);
   y[0] = 0;
   y[1] = b[0];
   for(i = 1;i < npoint;i++) {
      xav = 0.5*(a[i - 1] + a[i]);
      x[2*i] = x[2*i + 1] = xav;
      y[2*i] = b[i - 1];
      y[2*i + 1] = b[i];
   }
   xav = 0.5*(3*a[i - 1] - a[i - 2]);
   x[2*i] = x[2*i + 1] = xav;
   y[2*i] = b[i - 1];
   y[2*i + 1] = 0;		/* anchor last y point on axis */
   
   (void)sm_shade(delta,x,y,2*(npoint + 1));
   free((char *)x); free((char *)y);
   return(0);
}

/*****************************************************************/
/*
 * draw a line between (x0,y) and (x1,y) (rotated coordinates)
 */
static void
draw_seg(x0,x1,y)
float x0,x1,y;
{
   float yy0,yy1;			/* y0 and y1 are Bessel functions
					   that lint worries about */

   yy0 = y*cosa + x0*sina;		/* rotate back */
   x0 = x0*cosa - y*sina;
   yy1 = y*cosa + x1*sina;
   x1 = x1*cosa - y*sina;

   sm_line(x0,yy0,x1,yy1);
}

/*****************************************************************/
/*
 * Rotate a pair of vectors through -asin(sina) == -alpha
 */
static void
rotate(a,b,n)
float *a,*b;				/* arrays to rotate */
int n;					/* size of a,b */
{
   float x;
   
   while(--n >= 0) {
      x = a[n]*cosa + b[n]*sina;	/* rotate by -alpha */
      b[n] = -a[n]*sina + b[n]*cosa;
      a[n] = x;
   }
}
