#include "copyright.h"
/*
 * Here is a function to get various directories and files needed by SM
 */
#include "options.h"
#include <stdio.h>
#include <ctype.h>
#if defined(__MSDOS__)
#  include <dir.h>
#endif
#include "sm_declare.h"

#define NPATH 5				/* max number of elements on path */
#define SIZE 200

extern char cchar;			/* comment character */
#if defined(__MSDOS__)
#  define DEFAULTS_FILE "sm.rc"
#else
#  define DEFAULTS_FILE ".sm"
#endif
static char *defaults_file = DEFAULTS_FILE; /* file to look for defaults */
extern int sm_verbose;			/* should I be chatty? */
static char *path[NPATH + 1] = {	/* path to search for .sm files */
   "",
   "~",
   NULL,
};

void
set_defaults_file(file,home)
char *file;				/* .sm file */
char *home;				/* home directory */
{
   int i;
   char *ptr;
   static char *s_path = NULL;

   if(file != NULL) defaults_file = file;
   if(*defaults_file == '/') {		/* an absolute specification */
      path[0] = ""; path[1] = NULL;
      return;
   }

   if((ptr = getenv("SMPATH")) == NULL || *ptr == '\0') {
#if defined(DEFAULT_PATH)
      ptr = DEFAULT_PATH;
#endif
   }
   if(ptr == NULL || *ptr == '\0') {
      ptr = ". ~";
   }
   
   if(s_path != NULL) free(s_path);
   if((s_path = malloc(strlen(ptr) + 1)) == NULL) {
      fprintf(stderr,"Can't allocate storage for s_path\n");
      path[0] = ""; path[1] = NULL;
      return;
   }
   ptr = strcpy(s_path,ptr);
   for(i = 0;i < NPATH;i++) {
      path[i] = ptr;
      if((ptr = strchr(path[i],' ')) == NULL) {
	 break;
      }
      *ptr++ = '\0';
   }
   path[++i] = NULL;

   for(i = 0;path[i] != NULL;i++) {
#if defined(VMS)
      if(strcmp(path[i],".") == 0) {
	 path[i] = "";
      } else if(strcmp(path[i],"~") == 0) {
	 if(home == NULL) {
	    path[i] = "sys$login:";
	 }
      }
#else
#  if defined(__MSDOS__)
      if(strcmp(path[i],"~") == 0) {
         if(home == NULL) {
            extern char **_argv;
	    char drive[MAXPATH], dir[MAXDIR];
	    static char s_home[MAXPATH];
	 
	    fnsplit( _argv[0], drive, dir, NULL, NULL );
	    if((path[i] = malloc(strlen(dir)+3)) == NULL) {
	       msg("Can't allocate storage for home directory\n");
	       path[i] = "";
	       continue;
	    }
	    (void)sprintf(s_home,"%s%s",drive,dir);
	    home = s_home;
	 }
	 path[i] = home;
      }
#  else
      if(strcmp(path[i],"~") == 0) {
	 if(home == NULL) {
	    if((home = getenv("HOME")) == NULL) {
	       msg("Can't find home directory\n");
	       path[i] = "";
	       continue;
	    }
	 }
	 if((path[i] = malloc(strlen(home) + 2)) == NULL) {
	    msg("Can't allocate storage for home directory\n");
	    path[i] = "";
	    continue;
	 }
	 (void)sprintf(path[i],"%s/",home);
      }
#  endif
#endif
   }
}

char *
get_val(what)
char what[];			/* desired property */
{
   char file[100],
	in_line[SIZE],			/* line from file */
	name[SIZE],			/* name of attribute */
	*vptr;				/* pointer to value */
   static char value[SIZE];		/* the desired value */
   FILE *fil;
   int concatenate;			/* concatenate values from .sm files?*/
   int i,j,k;
   static int found = 0,		/* have we found a .sm file? */
	      warned = 0;		/* have we moaned about !found? */

   vptr = value;
   for(k = 0;path[k] != NULL;k++) {
      if(strcmp(path[k],".") == 0) {
	 strcpy(file,defaults_file);
      } else {
	 (void)sprintf(file,"%s%s",path[k],defaults_file);
      }
      if((fil = fopen(file,"r")) == NULL) {
	 continue;
      }
      found = 1;			/* we found a .sm file */

      concatenate = 0;
      while(fgets(in_line,SIZE,fil) != NULL) {
	 if(in_line[0] == cchar) {		/* a comment */
	    continue;
	 } else if(in_line[0] == '@' || in_line[0] == '+') {
	    (void)str_scanf(&in_line[1],"%s",name);
	 } else {
	    (void)str_scanf(in_line,"%s",name);
	 }
	 if(strcmp(name,what) == 0) {
	    if(in_line[0] == '@') {	/* this entry is explicitly missing */
	       (void)fclose(fil);
	       return(vptr > value ? value : NULL);
	    } else if(in_line[0] == '+') {
	       concatenate = 1;
	    }

	    i = strlen(in_line) - 1;
	    if(in_line[i] == '\n') in_line[i] = '\0'; /* remove \n */
	    
	    for(i = 0;i < SIZE && in_line[i] != '\0' &&
		!isspace(in_line[i]);i++) ;	/* skip name */
	    for(;i < SIZE && isspace(in_line[i]);i++) ;	/* find value */
	    for(j = i;in_line[j] != '\0';j++) {
	       if(in_line[j] == cchar) {
		  j--;
		  if(j > i) {
		     while(isspace(in_line[j])) j--;
		  }
		  in_line[++j] = '\0';
		  break;
	       } else if(in_line[j] == '\\') {
		  if(in_line[j+1] == '\\' || in_line[j+1] == cchar) {
		     if(in_line[j + 2] != '\0') {
			j++;		/* skip escaped characters */
		     }
		  }
	       }
	    }
	    if(in_line[i] == '\0' || i == SIZE || i == j) {
	       if(concatenate) {
		  msg("You must specify a value if you use +name in .sm\n");
	       }
	       (void)fclose(fil);
	       return("1");
	    } else {
	       if(vptr > value) *vptr++ = ' ';
	       j = strlen(&in_line[i]);
	       if(vptr + j > value + SIZE) {
		  msg_1s("Total value for %s in .sm is too long\n",what);
		  (void)fclose(fil);
		  return(vptr > value ? value : NULL);
	       }
	       strcpy(vptr,&in_line[i]);
	       vptr += j;
	       if(!concatenate) {
		  (void)fclose(fil);
		  return(value);
	       }
	    }
	 }
      }
      (void)fclose(fil);
   }
   if(!found && sm_verbose && !warned) {
      warned = 1;
      msg_1s("Can't find a file \"%s\"\n",defaults_file);
   }

   if(vptr > value) return(value);

   if(isupper(what[0])) {		/* Try the environment */
      return(getenv(what));
   }
   return(NULL);			/* not found */
}

/****************************************************/
/*
 * Add an entry to the defaults_file file
 */
int
put_val(name,value)
char name[],
     value[];
{
   char file[SIZE];
   FILE *fil;

   if((fil = fopen(defaults_file,"r+")) == NULL) {
#if defined(VMS)
      (void)sprintf(file,"sys$login:%s",defaults_file);
#else
#  if defined(__MSDOS__)
      (void)sprintf(file,"./%s",defaults_file);
#  else
      (void)sprintf(file,"%s/%s",getenv("HOME"),defaults_file);
#  endif
#endif
      if((fil = fopen(file,"r+")) == NULL) {
	 if((fil = fopen(file,"w")) == NULL) {
	    msg_1s("I'm sorry, but I can't create a %s file for you\n",
		   DEFAULTS_FILE);
	    return(-1);
	 }
      }
   }
   fseek(fil,0L,2);
   fprintf(fil,"%s\t\t%s",name,value);
   (void)fclose(fil);
   return(0);
}
