#include "copyright.h"
/*
 * This file contains a number of functions to do misc. operations on vectors.
 *
 *    function			finds
 *
 * vec_subscript(vec,ind,v)	vec[ind] = v
 */
#include "options.h"
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "vectors.h"
#include "sm_declare.h"

extern int sm_interrupt,			/* Have we seen a ^C? */
  	   sm_verbose;			/* shall we be chatty? */

/***********************************************************/
/*
 * Find vec[ind] = v (i.e. replace some elements of vec with v)
 */
void
vec_subscript(x,ind,v)
VECTOR *x,*ind,*v;
{
   int first_err = 1;			/* the first error that we've found */
   int i;				/* index into i */
   int vi;				/* index into v */
   int xi;				/* index into x */
   
   if(ind->dimen != v->dimen && v->dimen != 1) {
      msg_2s("Vectors %s and %s have different sizes",
	     ind->name,v->name);
      yyerror("show where");
      vec_free(ind); vec_free(v);
      return;
   }
   if(ind->type == VEC_STRING) {
      msg_1s("Vector %s is not numeric",ind->name);
      yyerror("show where");
      vec_free(ind); vec_free(v);
      return;
   }
   if(x->type != VEC_STRING && v->type == VEC_STRING) {
      msg_1s("vector %s is string-valued\n",v->name);
      vec_free(ind); vec_free(v);
      return;
   }
/*
 * OK, we can do this.
 */
   for(i = 0;i < ind->dimen;i++) {
      xi = (int)(ind->vec.f[i]);
      vi = (v->dimen == 1 ? 0 : i);
      if(xi < 0 || xi >= x->dimen) {
	 msg_1d("Index %d ",xi);
	 msg_1s("for vector %s is out of range ",x->name);
	 msg_1d("0 - %d",x->dimen-1);
	 if(first_err) {
	    yyerror("show where");
	    first_err = 0;
	 } else {
	    msg("\n");
	 }
	 xi = (xi < 0) ? 0 : x->dimen - 1;
      }
      if(x->type == VEC_STRING) {
	 if(v->type == VEC_STRING) {
	    (void)strncpy(x->vec.s.s[xi],v->vec.s.s[vi],VEC_STR_SIZE);
	 } else {
	    (void)sprintf(x->vec.s.s[xi],"%g",v->vec.f[vi]);
	 }
      } else {
	 x->vec.f[xi] = v->vec.f[vi];
      }
   }
   vec_free(ind);
   vec_free(v);
}
